local Plugin = {'neovim/nvim-lspconfig'}

function Plugin.init()
	local opts = { noremap=true, silent=true }
	vim.keymap.set('n', '<C-x>', vim.diagnostic.open_float, opts)
	vim.keymap.set('n', '[d', vim.diagnostic.goto_prev, opts)
	vim.keymap.set('n', ']d', vim.diagnostic.goto_next, opts)
	vim.keymap.set('n', '<C-z>', vim.diagnostic.setloclist, opts)
end

function Plugin.config()
	local nvim_lsp = require 'lspconfig'

	local on_attach = function(client)
			-- Enable completion triggered by <c-x><c-o>
		-- vim.api.nvim_buf_set_option(bufnr, 'omnifunc', 'v:lua.vim.lsp.omnifunc')

		-- Mappings.
		-- See `:help vim.lsp.*` for documentation on any of the below functions
		local bufopts = { noremap=true, silent=true }
		vim.keymap.set('n', 'gD', vim.lsp.buf.declaration, bufopts)
		vim.keymap.set('n', 'gd', vim.lsp.buf.definition, bufopts)
		vim.keymap.set('n', 'K', vim.lsp.buf.hover, bufopts)
		vim.keymap.set('n', 'gi', vim.lsp.buf.implementation, bufopts)
		-- vim.keymap.set('n', '<C-k>', vim.lsp.buf.signature_help, bufopts)
		-- vim.keymap.set('n', '<space>wa', vim.lsp.buf.add_workspace_folder, bufopts)
		-- vim.keymap.set('n', '<space>wr', vim.lsp.buf.remove_workspace_folder, bufopts)
		-- vim.keymap.set('n', '<space>wl', function()
		-- 	print(vim.inspect(vim.lsp.buf.list_workspace_folders()))
		-- end, bufopts)
		-- vim.keymap.set('n', '<space>D', vim.lsp.buf.type_definition, bufopts)
		-- vim.keymap.set('n', '<space>rn', vim.lsp.buf.rename, bufopts)
		vim.keymap.set('n', '<leader>ca', vim.lsp.buf.code_action, bufopts)
		-- vim.keymap.set('n', 'gr', vim.lsp.buf.references, bufopts)
		vim.keymap.set('n', '<leader>g', function() vim.lsp.buf.format { async = true } end, bufopts)
	end

	local lsp_flags = {
		-- This is the default in Nvim 0.7+
		debounce_text_changes = 150,
	}

	nvim_lsp.anakin_language_server.setup({
		on_attach = on_attach,
		flags = lsp_flags,
	})

	nvim_lsp.csharp_ls.setup({
		on_attach = on_attach,
		flags = lsp_flags,
	})

	nvim_lsp.jdtls.setup({
		on_attach = on_attach,
		flags = lsp_flags
	})

	nvim_lsp.tsserver.setup({
		on_attach = on_attach,
		flags = lsp_flags,
	})

	nvim_lsp.lua_ls.setup({
		on_attach = on_attach,
		flags = lsp_flags,
	})

	nvim_lsp.rust_analyzer.setup({
			on_attach=on_attach,
			flags = lsp_flags,
			settings = {
					["rust-analyzer"] = {
							-- checkOnSave = {
							-- 	command = "clippy"
							-- },
							imports = {
									granularity = {
											group = "module",
									},
									prefix = "self",
							},
							cargo = {
									autoreload = true,
									buildScripts = {
											enable = true,
									},
							},
							procMacro = {
									enable = true
							},
					}
			}
	})

end


return Plugin
